<?php
/**
 * EAN for WooCommerce - Core Class
 *
 * @version 1.3.0
 * @since   1.0.0
 * @author  Algoritmika Ltd
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_EAN_Core' ) ) :

class Alg_WC_EAN_Core {

	/**
	 * Constructor.
	 *
	 * @version 1.3.0
	 * @since   1.0.0
	 * @todo    [next] Order Items Table: move this to another file
	 * @todo    [later] search by EAN in "New Order"
	 * @todo    [later] check for duplicated EAN
	 */
	function __construct() {
		if ( 'yes' === get_option( 'alg_wc_ean_plugin_enabled', 'yes' ) ) {
			// Properties
			$this->ean_key = '_alg_ean';
			// Core
			require_once( 'class-alg-wc-ean-' . ( is_admin() ? 'backend' : 'frontend' ) . '.php' );
			// Order Items Table
			if ( 'yes' === get_option( 'alg_wc_ean_order_items_table', 'no' ) ) {
				add_action( 'woocommerce_order_item_meta_end', array( $this, 'add_to_order_item_meta_ean' ), 10, 4 );
			}
		}
		// Core loaded
		do_action( 'alg_wc_ean_core_loaded', $this );
	}

	/**
	 * get_ean_from_order_item.
	 *
	 * @version 1.2.0
	 * @since   1.2.0
	 */
	function get_ean_from_order_item( $item ) {
		return ( is_a( $item, 'WC_Order_Item_Product' ) && ( ( $product_id = $item->get_variation_id() ) || ( $product_id = $item->get_product_id() ) ) &&
			'' !== ( $ean = $this->get_ean( $product_id ) ) ? $ean : false );
	}

	/**
	 * add_to_order_item_meta_ean.
	 *
	 * @version 1.2.0
	 * @since   1.2.0
	 * @todo    [later] customizable template(s)
	 */
	function add_to_order_item_meta_ean( $item_id, $item, $order, $plain_text ) {
		if ( false !== ( $ean = $this->get_ean_from_order_item( $item ) ) ) {
			if ( ! $plain_text ) {
				echo '<ul class="wc-item-meta"><li>' .
					'<span class="sku_wrapper ean_wrapper">' . esc_html__( 'EAN:', 'ean-for-woocommerce' ) . ' ' .
						'<span class="ean">' . $ean . '</span>' .
					'</span>' .
				'</li></ul>';
			} else {
				echo "\n- " . esc_html__( 'EAN:', 'ean-for-woocommerce' ) . ' ' . $ean;
			}
		}
	}

	/**
	 * is_valid_ean.
	 *
	 * @version 1.0.1
	 * @since   1.0.1
	 * @see     https://stackoverflow.com/questions/19890144/generate-valid-ean13-in-php
	 * @todo    [next] add more formats, e.g. EAN-8?
	 * @todo    [maybe] `while ( strlen( $ean ) < 13 ) { $ean = '0' . $ean; }`
	 */
	function is_valid_ean( $ean ) {
		/**
		 * Valid   EAN-13 example: 5901234123457
		 * Invalid EAN-13 example: 1234567891789
		 */
		if ( 13 != strlen( $ean ) ) {
			return false;
		}
		$ean = preg_split( "//", $ean, -1, PREG_SPLIT_NO_EMPTY );
		$a = $b = 0;
		for ( $i = 0; $i < 6; $i++ ) {
			$a += ( int ) array_shift( $ean );
			$b += ( int ) array_shift( $ean );
		}
		$total = ( $a * 1 ) + ( $b * 3 );
		$next_ten = ceil( $total / 10 ) * 10;
		return ( $next_ten - $total == array_shift( $ean ) );
	}

	/**
	 * get_ean.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [later] use `$do_try_parent`
	 * @todo    [maybe] rethink `$product_id`
	 */
	function get_ean( $product_id = false, $do_try_parent = false ) {
		if ( ! $product_id ) {
			$product_id = get_the_ID();
		}
		$ean = get_post_meta( $product_id, $this->ean_key, true );
		if ( '' === $ean && $do_try_parent && 0 != ( $parent_id = wp_get_post_parent_id( $product_id ) ) ) {
			$ean = get_post_meta( $parent_id, $this->ean_key, true );
		}
		return $ean;
	}

}

endif;

return new Alg_WC_EAN_Core();
